@@ -35,7 +35,7 @@ __copyright__ = 'Copyright 2014-2019 - Block Research Group, ETH Zurich'
 __license__ = 'MIT License'
 __email__ = 'vanmelet@ethz.ch'
 
-__version__ = '0.16.2'
+__version__ = '0.16.3'
 
 
 PY3 = sys.version_info[0] == 3
@@ -72,12 +72,7 @@ except Exception:
     pass
 
 
-__all__ = [
-    'raise_if_windows',
-    'raise_if_not_windows',
-    'raise_if_ironpython',
-    'raise_if_not_ironpython',
-]
+__all__ = ['WINDOWS', 'LINUX', 'MONO', 'IPY', 'RHINO', 'GH', 'BLENDER', 'set_precision', 'get']
 
 
 def is_windows():
@@ -152,36 +147,16 @@ def is_rhino():
 RHINO = is_rhino()
 
 
-def is_grasshopper():
+def is_blender():
     try:
-        import Grasshopper  # noqa : F401
+        import bpy  # noqa : F401
     except ImportError:
         return False
     else:
         return True
 
 
-GH = is_grasshopper()
-
-
-def raise_if_not_windows():
-    if not WINDOWS:
-        raise
-
-
-def raise_if_windows():
-    if WINDOWS:
-        raise
-
-
-def raise_if_not_ironpython():
-    if not IPY:
-        raise
-
-
-def raise_if_ironpython():
-    if IPY:
-        raise
+BLENDER = is_blender()
 
 
 def set_precision(precision):
